--
-- GROUP and USER
--

/* create group so that we can test the IN GROUP attribute in CREATE USER */
CREATE GROUP rhdbgroup1;

/* CREATE USER tests */
CREATE USER rhdb1;

CREATE USER rhdb2 with SYSID 4321;

CREATE USER rhdb3 with SYSID 4322 NOCREATEDB;

CREATE USER rhdb4 with SYSID 4323 NOCREATEDB NOCREATEUSER;

CREATE USER rhdb5 with SYSID 4324 CREATEDB;

CREATE USER rhdb6 with SYSID 4326 CREATEUSER;

CREATE USER rhdb7 with IN GROUP rhdbgroup1;

CREATE USER rhdb8 with VALID UNTIL 'Jan 1, 1970 23:59:59';

CREATE USER rhdb9 with VALID UNTIL 'Dec 31, 2500 00:00:00';

CREATE USER rhdb10;

/* CREATE USER tests with invalid arguments */
CREATE USER rhdb110 with SYSID -111; -- error: invalid sysid

CREATE USER rhdb111 with IN GROUP rhdbgroup2; -- error: group doesn't exist

CREATE USER rhdb112 with GARBAGE; -- error: invalid attribute

CREATE USER rhdb1; -- error: user already exist

/* CREATE GROUP tests */
CREATE GROUP rhdbgroup2;

CREATE GROUP rhdbgroup3 with SYSID 800;

CREATE GROUP rhdbgroup4 with SYSID 811 USER rhdb1, rhdb2;

/* CREATE GROUP tests with invalid arguments */
CREATE GROUP rhdbgroup2; -- error: group already exist

CREATE GROUP rhdbgroup101 with SYSID xxx; -- error: invalid group id

CREATE GROUP rhdbgroup101 with USER xxx, rhdb1; -- error: user doesn't exist

/* ALTER GROUP tests (ADD USER) */
ALTER GROUP rhdbgroup2 ADD USER rhdb1;

ALTER GROUP rhdbgroup3 ADD USER rhdb10, rhdb9, rhdb8, rhdb7, rhdb6, rhdb5, rhdb4, rhdb3, rhdb2, rhdb1;

ALTER GROUP rhdbgroup4 ADD USER rhdb9, rhdb7, rhdb5, rhdb3;

/* ALTER GROUP tests with invalid arguments */
ALTER GROUP rhdbgroup2; -- error: incorrect syntax

ALTER GROUP rhdbgroup4 ADD rhdb9, rhdb7, rhdb5, rhdb3, rhdb1;
-- error: incorrect syntax

ALTER GROUP rhdbgroup2 ADD USER rhdb101; -- error: user doesn't exist

ALTER GROUP rhdbgroup3 ADD USER rhdb1; -- error: user already in group
 

/* CREATE TABLE/VIEW/SEQUENCE for tests on GRANT and REVOKE */
CREATE TABLE table1 (
   i1   integer,
   i2   integer,
   i3   integer,
   i4   integer,
   i5   integer,
   i6   integer
);

CREATE TABLE table2 (
   t1   text,
   t2   text
);

INSERT INTO table1 VALUES (1,2,3,4,5,6);
INSERT INTO table1 VALUES (11,12,13,14,15,16);
INSERT INTO table1 VALUES (21,22,23,24,25,26);
INSERT INTO table1 VALUES (31,32,33,34,35,36);
INSERT INTO table1 VALUES (41,42,43,44,45,46);
INSERT INTO table1 VALUES (51,52,53,54,55,56);

INSERT INTO table2 VALUES ('abc','def');
INSERT INTO table2 VALUES ('ghi','jkl');
INSERT INTO table2 VALUES ('mno','pqr');
INSERT INTO table2 VALUES ('stu','vwy');

CREATE VIEW view1 AS SELECT * FROM table1 CROSS JOIN table2;

CREATE SEQUENCE seq1 INCREMENT 3 MINVALUE -10 MAXVALUE 1000;

CREATE INDEX index1 ON table1 (i1);

/* GRANT tests */
GRANT ALL ON table1 TO rhdb1;

GRANT UPDATE, SELECT ON table1, table2 TO rhdb1;

GRANT INSERT, DELETE ON table1 TO rhdb2;

GRANT INSERT, DELETE, RULE, UPDATE, SELECT ON table1, view1, seq1 TO rhdb5;

GRANT SELECT, INSERT, UPDATE, DELETE, RULE ON table1, view1, seq1 TO PUBLIC;

GRANT UPDATE, SELECT, INSERT, DELETE, RULE, UPDATE ON table2 TO GROUP rhdbgroup1;
/* GRANT tests with invalid argument */
GRANT UPDATE, SELECT, INSERT, DELETE, RULE, ALL ON table2 TO GROUP rhdbgroup1;
-- mixing ALL with others

GRANT INSERT, DELETE ON index1 TO rhdb2;
-- try to grant privilege on an index

GRANT INSERT, DELETE ON something TO rhdb2; -- unknown relation

GRANT INSERT, DELETE ON table1 TO someone; -- unknown user


/* REVOKE tests */
REVOKE ALL ON table1 FROM rhdb1;

REVOKE UPDATE ON table2 FROM GROUP rhdbgroup1;

REVOKE ALL ON table1, view1, seq1 FROM PUBLIC;

REVOKE UPDATE ON table2 FROM GROUP rhdbgroup1;
REVOKE SELECT ON table2 FROM GROUP rhdbgroup1;
REVOKE INSERT ON table2 FROM GROUP rhdbgroup1;
REVOKE DELETE ON table2 FROM GROUP rhdbgroup1;
REVOKE RULE ON table2 FROM GROUP rhdbgroup1;

/* REVOKE tests with invalid argument */

REVOKE ALL ON table1 FROM rhdb111; -- error: user doesn't exist
REVOKE SOMETHING ON table1 FROM rhdb1; -- error: unknown priviledge
REVOKE ALL ON seq1 FROM rhdb9; -- error: user doesn't have that priviledge


/* ALTER GROUP tests (DROP USER) */
ALTER GROUP rhdbgroup2 DROP USER rhdb1;

ALTER GROUP rhdbgroup4 DROP USER rhdb1, rhdb3, rhdb7, rhdb9;

/* ALTER GROUP tests with invalid arguments */
ALTER GROUP rhdbgroup2 DROP USER rhdb1; -- error: user doesn't belong to the group anymore

ALTER GROUP rhdbgroup3 DROP USER rhdb1, rhdb3, rhdb7, rhdb9, rhdb2, rhdb4, rhdb6, rhdb8, rhdb10, rhdb5, rhdb111; -- error: the last user doesn't exist



/* just to make sure we can add these users back to the group after they are dropped */
ALTER GROUP rhdbgroup3 ADD USER rhdb10, rhdb9, rhdb8, rhdb7, rhdb6, rhdb5, rhdb4, rhdb3, rhdb2, rhdb1;

-- cleanup
-- drop view
DROP VIEW view1;

-- drop sequence
DROP SEQUENCE seq1;

-- drop table
DROP TABLE table1;

DROP TABLE table2;

-- drop group
DROP GROUP rhdbgroup1;

DROP GROUP rhdbgroup2;

DROP GROUP rhdbgroup3;

DROP GROUP rhdbgroup4;


DROP GROUP rhdbgroup101; -- error: group doesn't not exist

-- drop user
DROP USER rhdb1;

DROP USER rhdb2;

DROP USER rhdb3;

DROP USER rhdb4;

DROP USER rhdb5;

DROP USER rhdb6;

DROP USER rhdb7;

DROP USER rhdb8;

DROP USER rhdb9;

DROP USER rhdb10;

DROP USER rhdb20;

DROP USER rhdb101; -- error: this user doesn't exist

